#!/usr/bin/env python
# -*- coding: utf-8 -*-

### GAD PIPELINE ###
## LabkeyTools.py
## Description : Module with functions to interact with the GAD's Labkey database
## Usage : import LabkeyTools
## Output : no output
## Requirements : Python 2.7+ Supported formats: .bed, .interval_list and .list

## Author : jbjriviere@gmail.com
## Creation Date : 29/07/2015
## Last revision date : 04/04/2019
## Known bugs : None


import labkey
import sys
import time
from datetime import date
from datetime import timedelta
import ssl 
ssl._create_default_https_context = ssl._create_unverified_context



usage = """
Please read the documentation for a proper use !
"""

def formatDate( incArg ):
    incDate = incArg; 
    tempDate = incDate.split( " " )[0]
    outDate = tempDate.replace( '/' , '-' )
    return outDate


def officialDate( incArg ):
	return date( int(incArg.split( "-" )[0]) , int(incArg.split( "-" )[1])  , int(incArg.split( "-" )[2]) )


def GetRows(arg):
	"""Returns a dictionary of a given query in Labkey. The 'rows' key is  a dictionary of lines"""
	# case of coverage and multiple values
	if( arg == "MiSeq coverage" ):
		# ~ sys.stderr.write( "Using the %s dataset\n" % arg )
		return labkey.query.selectRows(baseUrl = "https://translad.chu-dijon.fr/labkey", containerPath = "project/home/GAD/Génétique moléculaire", schemaName = "study", queryName = arg , sort = "Date" )
	elif( arg == "Exome coverage" ):
		# ~ sys.stderr.write( "Using the %s dataset\n" % arg )
		return labkey.query.selectRows(baseUrl = "https://translad.chu-dijon.fr/labkey", containerPath = "project/home/GAD/Génétique moléculaire", schemaName = "study", queryName = arg , sort = "date" )
	elif( arg == "Suivi exomes" ):
		# ~ sys.stderr.write( "Using the %s dataset\n" % arg )
		return labkey.query.selectRows(baseUrl = "https://translad.chu-dijon.fr/labkey", containerPath = "project/home/GAD/Génétique moléculaire", schemaName = "study", queryName = arg , sort = "date_analyse" )
	elif( arg == "find_coverage" ):
		# ~ sys.stderr.write( "Using the %s dataset\n" % arg )
		return labkey.query.selectRows(baseUrl = "https://translad.chu-dijon.fr/labkey", containerPath = "project/home/GAD/Génétique moléculaire", schemaName = "study", queryName = arg , sort = "date" )
	else:
		# ~ sys.stderr.write( "Using the %s dataset (default)\n" % arg )	
		return labkey.query.selectRows(baseUrl = "https://translad.chu-dijon.fr/labkey", containerPath = "project/home/GAD/Génétique moléculaire", schemaName = "study", queryName = arg)


def GetList(arg):
	"""Returns a dictionary of a given query in Labkey. The 'rows' key is  a dictionary of lines"""
	return labkey.query.selectRows(baseUrl = "https://translad.chu-dijon.fr/labkey", containerPath = "project/home/GAD/Génétique moléculaire", schemaName = "lists", queryName = arg)

# Classes
class PatientReport:
	"""
	Patient information with several features related to a molecular report
	WARNING: only one line per patient allowed in each Labkey dataset"""
	def __init__(self, arg , argTwo):
		"""..."""
		
		# verify ped reception
		if not arg:
			sys.exit("%s\nErreur: Vous devez spécifier un identifiant PED id à l'aide de l'option -p\n" % usage )
		
		self.ID = arg
		self.Rank = int(argTwo)

		# Compte-rendus
		try:
			self.Report = [l for l in GetRows("Compte-rendus")["rows"] if l["PatientID"] == self.ID and l["progress"] == "A rédiger" ]
		except NameError:
			sys.exit("%s\nError: %s not found in Compte-rendus à rédiger" % (usage, self.ID))
        
        
        # listes de cohortes 
		allCohortes = labkey.query.selectRows(baseUrl = "https://translad.chu-dijon.fr/labkey", containerPath = "project/home/GAD/Génétique moléculaire", schemaName = "lists", queryName = "Sous-Cohortes" )
		self.cohortesList = {}
		for line in allCohortes["rows"]:
			self.cohortesList[ str( line["Key"] ) ] = line["Nom"] 
        
        
        
                
		# multiple CR or not CR : 
		# seems to not reach this step when 0 CR is found. 
		# added a bloc before for controling answer of GetRows
		if len(self.Report) == 0:
			sys.exit("%s\nError: %s n'est pas dans les Compte-rendus à rédiger" % (usage, self.ID))
		elif len(self.Report) > 1:
			sys.exit("%s\nError: %s trouvé %s fois dans les Compte-rendus à rédiger" % (usage, self.ID, len(self.Report)))
			
		self.title = { "Anne-Laure Mosca-Boidron" : "Dr" , "Frédéric Tran Mau-Them" : "Dr" , "Paul Kuentz" : "Dr" , "Julien Thevenon" : "Dr" ,  "Yannis Duffourd" : "M." , "Emilie Tisserant" : "Dr ès sc." , "Patrick Callier" : "Pr" , "Christophe Philippe" : "Pr" , "Christel Thauvin" : "Pr" , "Laurence Faivre" : "Pr" , "Daphné Lehalle" : "Dr" , "Nolwenn Jean" : "Dr" , "Antonio Vitobello" : "Dr ès sc." , "Ange-Line Bruel" : "Dr ès sc." , "Virginie Carmignac" : "Dr ès sc." , "Pierre Vabres" : "Pr" , "Sébastien Moutton" : "Dr" , "François Lecoquierre" : "M." , "Arthur Sorlin" : "Dr" , "Mirna Assoum" : "Dr ès sc." , "Geoffroy Delplancq" : "M." , "Julian Delanne" : "Dr." , "Anne-Sophie Denommé-Pichon" : "Dr" , "Mathilde Lefebvre" : "Dr" , "Nicolas Bourgon" : "M." , "Aurélien Juven" : "M." , "Aurore Garde" : "Mme" , "Melchior de Giraud D'Agay": "M." , "Caroline Racine" : "Mme", "Camille Engel" : "Mme" , "Alexis Overs" : "M." ,  "Hana Safraou" : "Dr" , "Mylène Tharreau" : "Mme" }
		
		#self.signature = { "Anne-Laure Mosca-Boidron" : "Anne-Laure.Mosca-Boidron.jpg" , "Frédéric Tran Mau-Them" : "Frédéric.Tran-Mau-Them.png" , "Paul Kuentz" : "Paul.Kuentz.jpg" , "Julien Thevenon" : "signature_julien.jpg" ,  "Yannis Duffourd" : "Yannis.Duffourd.signature.jpg" , "Emilie Tisserant" : "Emilie.Tisserant.signature.png" , "Patrick Callier" : "Patrick.Callier.jpg" , "Christophe Philippe" : "Christophe.Philippe.jpg" , "Christel Thauvin" : "Christel.Thauvin.bmp" , "Laurence Faivre" : "Laurence.Faivre.png" , "Daphné Lehalle" : "Daphné.Lehalle.jpg" , "Nolwenn Jean" : "Dr" , "Antonio Vitobello" : "Antonio.Vitobello.signature.png" , "Ange-Line Bruel" : "Ange-Line.Bruel.signature.png" , "Virginie Carmignac" : "Dr" }

		self.signature = { "Anne-Laure Mosca-Boidron" : "/work/gad/shared/pipeline/utils/CR_utils/Anne-Laure.Mosca-Boidron.jpg" , "Frédéric Tran Mau-Them" : "/work/gad/shared/pipeline/utils/CR_utils/Frédéric.Tran-Mau-Them.png" , "Paul Kuentz" : "/work/gad/shared/pipeline/utils/CR_utils/Paul.Kuentz.jpg" , "Julien Thevenon" : "/work/gad/shared/pipeline/utils/CR_utils/signature_julien.jpg" ,  "Yannis Duffourd" : "/work/gad/shared/pipeline/utils/CR_utils/Yannis.Duffourd.signature.jpg" , "Emilie Tisserant" : "/work/gad/shared/pipeline/utils/CR_utils/Emilie.Tisserant.signature.png" , "Patrick Callier" : "/work/gad/shared/pipeline/utils/CR_utils/Patrick.Callier.jpg" , "Christophe Philippe" : "/work/gad/shared/pipeline/utils/CR_utils/Christophe.Philippe.jpg" , "Christel Thauvin" : "/work/gad/shared/pipeline/utils/CR_utils/Christel.Thauvin.bmp" , "Laurence Faivre" : "/work/gad/shared/pipeline/utils/CR_utils/Laurence.Faivre.png" , "Daphné Lehalle" : "/work/gad/shared/pipeline/utils/CR_utils/Daphné.Lehalle.jpg" , "Nolwenn Jean" : "Nolwenn.Jean.signature.png" , "Antonio Vitobello" : "/work/gad/shared/pipeline/utils/CR_utils/Antonio.Vitobello.signature.png" , "Ange-Line Bruel" : "/work/gad/shared/pipeline/utils/CR_utils/Ange-Line.Bruel.signature.png" , "Virginie Carmignac" : "/work/gad/shared/pipeline/utils/CR_utils/Virginie.Carmignac.signature.png" , "Pierre Vabres" : "/work/gad/shared/pipeline/utils/CR_utils/Pierre.Vabres.signature.png" , "Sébastien Moutton" : "/work/gad/shared/pipeline/utils/CR_utils/Sebastien.Moutton.signature.png" , "François Lecoquierre" : "/work/gad/shared/pipeline/utils/CR_utils/Francois.Lecoquierre.signature.png"  , "Arthur Sorlin" : "/work/gad/shared/pipeline/utils/CR_utils/Arthur.Sorlin.signature.jpg" , "Mirna Assoum" : "/work/gad/shared/pipeline/utils/CR_utils/Mirna.Assoum.signature.jpg" , "Geoffroy Delplancq" : "/work/gad/shared/pipeline/utils/CR_utils/Geoffroy.Delplancq.signature.png" , "Julian Delanne" : "/work/gad/shared/pipeline/utils/CR_utils/Julian.Delanne.signature.png" , "Anne-Sophie Denommé-Pichon" : "/work/gad/shared/pipeline/utils/CR_utils/Anne-Sophie.Denommé-Pichon.signature.png" , "Mathilde Lefebvre" : "/work/gad/shared/pipeline/utils/CR_utils/Mathilde.Lefebvre.signature.png" , "Nicolas Bourgon" : "/work/gad/shared/pipeline/utils/CR_utils/Nicolas.Bourgon.signature.png" , "Aurélien Juven" : "/work/gad/shared/pipeline/utils/CR_utils/Aurelien.Juven.signature.png" , "Aurore Garde" : "/work/gad/shared/pipeline/utils/CR_utils/Aurore.Garde.signature.jpg" , "Melchior de Giraud D'Agay": "/work/gad/shared/pipeline/utils/CR_utils/Melchior.De.Giraud.D.Agay.signature.png" , "Caroline Racine" : "/work/gad/shared/pipeline/utils/CR_utils/Caroline.Racine.signature.png", "Camille Engel" : "/work/gad/shared/pipeline/utils/CR_utils/Camille.Engel.signature.png" , "Alexis Overs" : "/work/gad/shared/pipeline/utils/CR_utils/Alexis.Overs.signature.png" , "Hana Safraou" : "/work/gad/shared/pipeline/utils/CR_utils/Hana.Safraou.signature.png" , "Mylène Tharreau" : "/work/gad/shared/pipeline/utils/CR_utils/Mylene.Tharreau.signature.png" }

		self.fonction = { "Anne-Laure Mosca-Boidron" : "Généticienne Praticien agréé" , "Frédéric Tran Mau-Them" : "Généticien" , "Paul Kuentz" : "Généticien Praticien agréé" , "Julien Thevenon" : "Généticien" ,  "Yannis Duffourd" : "Bioinformaticien" , "Emilie Tisserant" : "Bioinformaticienne" , "Patrick Callier" : "Généticien Praticien agréé" , "Christophe Philippe" : "Généticien Praticien agréé" , "Christel Thauvin" : "Généticienne" , "Laurence Faivre" : "Généticienne" , "Daphné Lehalle" : "Généticienne" , "Nolwenn Jean" : "Généticienne" , "Antonio Vitobello" : "Ingénieur" , "Ange-Line Bruel" : "Ingénieure" , "Virginie Carmignac" : "Ingénieure", "Pierre Vabres" : "Dermatologue" , "Sébastien Moutton" : "Généticien" , "François Lecoquierre" : "Interne" , "Arthur Sorlin" : "Généticien" , "Mirna Assoum" : "Ingénieure" , "Geoffroy Delplancq" : "Interne" , "Julian Delanne" : "Généticien" , "Anne-Sophie Denommé-Pichon" : "Généticienne Praticien agréé" ,  "Mathilde Lefebvre" : "Généticienne" ,  "Nicolas Bourgon" : "Interne" , "Aurélien Juven" : "Interne" , "Aurore Garde" : "Interne" , "Melchior de Giraud D'Agay": "Interne" , "Caroline Racine" : "Interne", "Camille Engel" : "Interne" , "Alexis Overs" : "Interne" , "Hana Safraou" : "Practicien Attaché Associé" , "Mylène Tharreau" : "Interne"}
				
		self.AnalysisDate = ""
		self.PipelineVersion = ""
		self.CaptureKit = ""
		self.Sequencer = ""
		self.Sample = str(self.Report[0]["SampleID"])
		self.Category = self.Report[0]["categorie"] 
		self.Exam = self.Report[0]["examen"]
		self.Result = self.Report[0]["resultat"]
		self.Biol = ""
		self.ClinicienL = ""
		self.Parente = ""
		
		if self.Exam.startswith( "Exome" ):
			self.Test = "Exome"
		elif self.Exam.startswith("Confirmation"):
			self.Test = "Confirmation"
		elif self.Exam.startswith( "qPCR" ):
			self.Test = "qPCR"
		elif self.Exam.startswith( "Genome" ):
			self.Test = "Genome"
			self.Exam = "Genome" 
		else:
			self.Test = "Miseq"
	
		
		
		
		ccl = GetList("Compte-rendus - Conclusion")
		ccl = {l["Result"].split(" - ")[0] : l["Result"].split(" - ")[1] for l in ccl["rows"]}
		self.FormatResult = "%s - %s" % (self.Result, ccl[self.Result])
		self.Conclusion = ""

		# get recommandation
		recom = GetList("Compte-rendus - Recommandations")
		sys.stderr.write( "categorie : %s\n" % self.Category )
		sys.stderr.write( "\ttest : %s\n" % self.Test )
		sys.stderr.write( "\tresult : %s\n" % self.Result )
		sys.stderr.write( "Searching in table recommandations ... \n" )
		for line in recom["rows"]:
			# ~ sys.stderr.write( "testing category : %s vs %s\n" % ( line["Categorie"]  , self.Category ) )
			if line["Categorie"] == self.Category:
				# ~ sys.stderr.write( "\tFound matching category\n" )
				res = line["Result"]
				
				if self.Category == "Diagnostic":
					if res == self.Result :
						# ~ sys.stderr.write( "\t\tresult : %s\n" % self.Result )
						if self.Test == "Exome" or self.Test == "Genome":
							self.Conclusion = line["Exome"]
						else:
							self.Conclusion = line["Autre"]
				else:
					res = res.split( "-" )[1]
					res = res.replace( " " , "" , 1 )
					# ~ sys.stderr.write( "testing result : %s vs %s\n" % ( res , self.Result ) )
					if res == self.Result:
						# ~ sys.stderr.write( "\tFound matching result\n" )
						# ~ sys.stderr.write( "testing Test : %s vs Exome/Autre\n" % self.Test )
						if self.Test == "Exome" or self.Test == "Genome":
							# ~ sys.stderr.write( "\tFound matching result : %s\n" % "Exome" )
							self.Conclusion = line["Exome"]
						else:
							# ~ sys.stderr.write( "\tFound matching result : %s\n" % "Autre" )
							self.Conclusion = line["Autre"]
		# ~ sys.stderr.write( "\t\t\tConclusion : %s\n" % self.Conclusion )
		
		if not hasattr(self, 'Conclusion'):
			sys.exit( "Conclusion non trouvée, container vide pour le patient\n" )
		if ( self.Conclusion is None or self.Conclusion == "" or self.Conclusion == "None" ) and ( self.Category != "Find" ) :
			sys.exit( "Conclusion non trouvée : None or empty (examen : %s , categorie : %s , resultat : %s)\n" % (  self.Test , self.Category ,  self.Result ) )
		
		if self.Category == "Find" :
			 self.Conclusion = " " 
		
		#  pipeline version 
		pipelineVersion = GetList("Pipeline version")

		
		self.Summary = self.Report[0]["interpretation"]
		self.Tech = self.Report[0]["technicien"]
		self.Bioinfo = self.Report[0]["bioinformaticien"]
		if not self.Exam.startswith("Exome") : 
			self.Biol = self.Report[0]["biologiste"]
		
		self.Sign = self.Report[0]["signataire"]
		self.Progress = self.Report[0]["progress"]
		
		if self.Exam == "Confirmation" or self.Exam == "qPCR": 
			#~ self.Biol = self.Sign
			self.ClinicienL = "Non renseigné"
		
		# Patients
		self.Patient = [l for l in GetRows("Patients")["rows"] if l["PatientID"] == self.ID]
		if len(self.Patient) == 0:
			sys.exit("%s\nError: %s not found in Patients" % (usage, pedID))
		elif len(self.Patient) > 1:
			sys.exit("%s\nError: %s found %s times in Patients" % (usage, pedID, len(self.Patient)))
		self.Last = self.Patient[0]["nom"]
		self.First = self.Patient[0]["prenom"]
		self.Birth = self.Patient[0]["date_de_naissance"]
		self.Sex = self.Patient[0]["sexe"]
		self.Clinician = self.Patient[0]["clinicien_referent"] 
		self.ClinicianShort = self.Clinician
		
		# manage multiple clinician 
		if (self.Clinician).find( "@" ) != -1 :
			self.ClinicianShort = (self.Clinician).split( "@" )[0]
			
		# getting Institution directly from the list of clinician, independantly of the view. 
		temp = [inst for inst in GetList("Patient - Clinicien")["rows"] if inst["clinicien"] == self.Clinician]
		self.Institution = temp[0]["institution"]
		
		# self.Institution = self.Patient[0]["clinicien_referent/institution"]
		# need to manage long institution names
		if len( self.Institution ) > 16:
		    temp = self.Institution.split( " " )
		    self.Institution = ""
		    for i in temp:
			if len( self.Institution ) > 16:
			    self.Institution += "\n" 
			self.Institution += i 
			self.Institution += " " 
		
		self.Parente = self.Patient[0]["parente"]
		
		
		tmp = self.Patient[0]["cohorte"]
		self.Cohort = self.cohortesList[str( tmp )] 
		
		
		self.Pathology = self.Patient[0]["pathologie"]
		if self.Pathology is None or self.Pathology == "" or self.Pathology == "Non renseigné" or self.Pathology == "None":
			sys.exit( "Erreur: %s : GAD %s : Pas de pathologie renseignée. Impossible de générer un compte-rendu." % ( self.ID , self.Sample ) )
		
		if "-" in self.Pathology:
			self.Pathology = self.Pathology.replace("-", " ")

		# DNAtheque
		self.DNA = [l for l in GetRows("DNAthèque")["rows"] if l["PatientID"] == self.ID and self.Sample == str(l["sampleID"])]
		if len(self.DNA) == 0:
			sys.exit("Error: %s : GAD %s n'est pas présent dans la DNAThèque" % ( self.ID , self.Sample))
		elif len(self.DNA) > 1:
			sys.exit("Error: %s found %s times in DNAtheque" % (usage, self.Sample, len(self.DNA)))
		
		# getting Origin directly from the list of reception, independantly of the view. 
		temp = [ori for ori in GetList("DNAthèque - Réception")["rows"] if str(ori["GAD"]).strip() == self.Sample]
		self.Origin = temp[0]["origine_de_l_echantillon"]
		
		self.Sampling = self.DNA[0]["date_de_prelevement"]
		
		# getting date directly from the list of reception, independantly of the view. 
		temp = [rdate for rdate in GetList("DNAthèque - Réception")["rows"] if str(rdate["GAD"]).strip() == self.Sample]
		self.Reception = temp[0]["date_de_reception"]

		# Coverage
		## For WES
		if self.Test is "Exome" or self.Test is "Genome" :
			if self.Category == "Find" :
				self.QC = [l for l in GetRows("find_coverage")["rows"] if l["PatientID"] == self.ID and self.Sample == str(l["SpecimenID"])]
			else:
				self.QC = [l for l in GetRows("Exome coverage")["rows"] if l["PatientID"] == self.ID and self.Sample == str(l["SpecimenID"])]
			if len(self.QC) == 0:
				sys.exit("Erreur: %s : GAD %s n'est pas présent dans la table de couverture." % (self.ID ,  self.Sample))
			else:
				indexForRecent = 0
				if len(self.QC) > 1:
					sys.stderr.write( "WARNING : %s Données de couverture trouvées pour l'échantillon %s" % ( len(self.QC) , self.Sample ) )
					sys.stderr.write( "\tUtilisation du plus récent.\n"  )
					
					# Check the last QC 
					oldDate = officialDate( "2000-01-01" ) 
					
					k = 0
					for elt in self.QC :
						if officialDate( elt["Run/date"] ) > oldDate:
							oldDate = officialDate( elt["Run/date"] )
							indexForRecent = k
						k += 1
					
				self.Depth = self.QC[indexForRecent]["mean"] 
				self.Cov10X = self.QC[indexForRecent]["bases_above_10"]
				self.Cov100X = self.QC[indexForRecent]["bases_above_100"]
				
				
				
				
		## For MiSeq
		elif self.Test is "Miseq":
			self.QC = [l for l in GetRows("MiSeq coverage")["rows"] if l["PatientID"] == self.ID and self.Sample == str(l["SpecimenID"]) and self.Exam == l["target"]]
			if len(self.QC) == 0:
				sys.exit("%s\nError: %s not found in MiSeq coverage" % (usage, self.Sample))
			else:
				if len(self.QC) > 1:
                                    sys.stderr.write( "WARNING : %s Données de couverture trouvées pour l'échantillon %s\n" % ( len(self.QC) , self.Sample ) )
                                    sys.stderr.write( "\tUtilisation du N°%s plus ancien\n" % self.Rank )				
				
				self.Depth = self.QC[self.Rank - 1]["mean"]
				self.Cov10X = self.QC[self.Rank - 1]["bases_above_10"]
				self.Cov100X = self.QC[self.Rank - 1]["bases_above_100"]
				self.Cov1000X = self.QC[self.Rank - 1]["bases_above_1000"]
                
		# Variants
		if self.Result is not "Négatif":
			"""Generates a list of dictionaries of individual variants"""
			self.Variants = [l for l in GetRows("Variations à vérifier")["rows"] if l["PatientID"] == self.ID and self.Sample == str(l["SampleID"]) and l["to_report"] == "Oui"]		
			
			# CNV Integration
			self.CNVs = [l for l in GetRows("CNV à vérifier")["rows"] if l["PatientID"] == self.ID and self.Sample == str(l["SampleID"]) and l["to_report"] == "Oui"]
			#sys.stderr.write( "%s\n" % self.Variants )
						
			
		
		# Check the informations are ok.
		# only for wes
		if self.Test is "Exome" or self.Test is "Genome":
			indexForRecentExome = 0
			self.SuiviExome = [l for l in GetRows("Suivi exomes")["rows"] if l["PatientID"] == self.ID and self.Sample == str(l["SpecimenID"]) ]
			if len( self.SuiviExome) == 0:
				sys.exit("%s\nError: %s not found in Suivi Exome" % (usage, self.Sample))
			if len( self.SuiviExome) > 1 :
				
				sys.stderr.write( "WARNING : %s données de suivi exome pour l'échantillon %s\n" % ( len(self.SuiviExome) , self.Sample ) )
				sys.stderr.write( "\tUtilisation de la lecture la plus récente : " )
				
				# Check the last wes analysis
				
				oldDate = officialDate( "2000-01-01" ) 
				
				k = -1
				for elt in self.SuiviExome :
					k += 1
					if elt["categorie"] == "Find" :
						continue
					if elt["date_lecture1"] is None  :
						continue
						# ~ sys.exit("Error: %s : GAD %s : n'a pas de date de lecture 1 : il n'est pas possible d'emettre un compte rendu si l'exome n'a pas été lu." % (self.ID, self.Sample))
					if elt["date_lecture2"] is None :
						continue
						# ~ sys.exit("Error: %s : GAD %s : n'a pas de date de lecture 2 : il n'est pas possible d'emettre un compte rendu si l'exome n'a pas été lu." % (self.ID, self.Sample))
					
					if officialDate( elt["date_lecture1"] ) > oldDate:
						oldDate = officialDate( elt["date_lecture1"] )
						indexForRecentExome = k
					if officialDate( elt["date_lecture2"] ) > oldDate:
						oldDate = officialDate( elt["date_lecture2"] )
						indexForRecentExome = k
				sys.stderr.write( "%s ; Résultat : %s\n" % (oldDate, elt["resultat"] ) )
				
				
				
				
			# check lectures are filled in
			if self.SuiviExome[indexForRecentExome]["lecture1"] is None or self.SuiviExome[indexForRecentExome]["lecture1"] == "":
				sys.exit("Error: %s : GAD %s : n'a pas de lecteur 1 : il n'est pas possible d'emettre un compte rendu si l'exome n'a pas été lu." % (self.ID, self.Sample))
			if self.SuiviExome[indexForRecentExome]["lecture2"] is None or self.SuiviExome[indexForRecentExome]["lecture2"] == "":
				sys.exit("Error: %s : GAD %s : n'a pas de lecteur 2 : il n'est pas possible d'emettre un compte rendu si l'exome n'a pas été lu." % (self.ID, self.Sample))
			if self.SuiviExome[indexForRecentExome]["date_lecture1"] is None or self.SuiviExome[indexForRecentExome]["date_lecture1"] == "":
				sys.exit("Error: %s : GAD %s : n'a pas de date de lecture 1 : il n'est pas possible d'emettre un compte rendu si l'exome n'a pas été lu." % (self.ID, self.Sample))
			if self.SuiviExome[indexForRecentExome]["date_lecture2"] is None or self.SuiviExome[indexForRecentExome]["date_lecture2"] == "":
				sys.exit("Error: %s : GAD %s : n'a pas de date de lecture 2: il n'est pas possible d'emettre un compte rendu si l'exome n'a pas été lu." % (self.ID, self.Sample))
			if self.SuiviExome[indexForRecentExome]["resultat"] is None or self.SuiviExome[indexForRecentExome]["resultat"] == "":
				sys.exit("Error: %s : GAD %s : n'a pas de résultat : il n'est pas possible d'emettre un compte rendu si l'exome n'a pas de résultat." % (self.ID, self.Sample))
			
			# get capture kit & sequencer
			if self.SuiviExome[indexForRecentExome]["sequencage"] is None or self.SuiviExome[indexForRecentExome]["sequencage"] == "":
				sys.exit("Error: %s : GAD %s : n'a pas de sequenceur : il n'est pas possible d'emettre un compte rendu si l'exome n'a pas été séquencé ." % (self.ID, self.Sample))
			else:
				self.Sequencer = self.SuiviExome[indexForRecentExome]["sequencage"]
			if self.SuiviExome[indexForRecentExome]["capture"] is None or self.SuiviExome[indexForRecentExome]["capture"] == "":
				sys.exit("Error: %s : GAD %s : n'a pas de capture : il n'est pas possible d'emettre un compte rendu si l'exome n'a pas été capturé ." % (self.ID, self.Sample))
			else:
				self.CaptureKit = self.SuiviExome[indexForRecentExome]["capture"]
			
			
			
			
			
			
			# check date coherence
			today = date.today()
			oldestDate = date( 3000 ,01 , 01 )
			# ~ sys.stderr.write( "result : %s\n" % self.Result )
			if not self.Result == "Négatif":
				for elt in self.Variants:
					tmp = str(elt["date_revision"].split( " " )[0])
					currentDate = date( int(tmp.split( "/" )[0]) , int(tmp.split( "/" )[1])  , int(tmp.split( "/" )[2]) )
					#sys.stderr.write( "V : %s\n" % currentDate )
					if currentDate < oldestDate:
						oldestDate = currentDate
				#sys.stderr.write( "V : %s\n" % oldestDate )
				#sys.stderr.write( "L2 : %s\n" % self.SuiviExome[self.Rank - 1]["date_lecture2"] )
				
				lec2Date = date( int(self.SuiviExome[indexForRecentExome]["date_lecture2"].split( "-")[0]) , int(self.SuiviExome[indexForRecentExome]["date_lecture2"].split( "-")[1]) , int(self.SuiviExome[indexForRecentExome]["date_lecture2"].split( "-")[2] ) )
				lec1Date = date( int( self.SuiviExome[indexForRecentExome]["date_lecture1"].split( "-")[0]) , int(self.SuiviExome[indexForRecentExome]["date_lecture1"].split( "-")[1] ) , int( self.SuiviExome[indexForRecentExome]["date_lecture1"].split( "-")[2] ))

				if oldestDate < lec1Date or oldestDate < lec2Date:
					sys.exit("Erreur: %s : GAD %s : Un problème a été détectée avec la date de lecture : il n'est pas possible de générer un compte-rendu lorsque la date de vérification d'une mutation est antérieure à la date de lecture." % (self.ID, self.Sample))
			
			
			# get analysis date
			if self.SuiviExome[indexForRecentExome]["date_analyse"] is None or self.SuiviExome[indexForRecentExome]["date_analyse"] == "":
				sys.exit("Error: %s : GAD %s : n'a pas de date d'analyse : il n'est pas possible d'emettre un compte rendu si l'exome n'a pas été analysé ." % (self.ID, self.Sample))
			else:
				self.AnalysisDate = date( int( self.SuiviExome[indexForRecentExome]["date_analyse"].split( "-")[0]) , int(self.SuiviExome[indexForRecentExome]["date_analyse"].split( "-")[1] ) , int( self.SuiviExome[indexForRecentExome]["date_analyse"].split( "-")[2] ))
			
			# infers pipeline version
			recent_date = date( 1980 , 01 , 01 )
			for line in pipelineVersion["rows"]:
				tmpDate = date( int( line["release_prod_date"].split( "-")[0] ) , int( line["release_prod_date"].split( "-")[1] ) , int( line["release_prod_date"].split( "-")[2] ))
				#~ sys.stderr.write( " date : %s ; analyse : %s\n" % ( tmpDate , self.AnalysisDate ) )
				if tmpDate < self.AnalysisDate :
					#~ sys.stderr.write( " OK inf \n" )
					if tmpDate > recent_date :
						recent_date = tmpDate
						#~ sys.stderr.write( " OK recent \n" )
						self.PipelineVersion = str( line["version"] )
			#~ sys.stderr.write( " vers : %s\n" % self.PipelineVersion )




			# third lecture
			if self.SuiviExome[indexForRecentExome]["lecture3"] is not None:
				self.ClinicienL = self.SuiviExome[indexForRecentExome]["lecture3"]
			else:
				self.ClinicienL = "Non renseigné"
			
			self.Biol = self.SuiviExome[indexForRecentExome]["lecture2"]
			self.Inge = self.SuiviExome[indexForRecentExome]["lecture1"]
			

class Variant:
	"""Generates an object with variant attributes from Labkey Variations à vérifier"""
	def __init__(self, arg):
		self.Gene = arg["gene"]
		self.Mut = "<br/>".join([arg["mut_genomic"], arg["mut_cdna"], arg["mut_aa"]])
		self.Inherit = arg["transmission"]
		self.Classification = arg["classification"]
		self.ToReport = arg["to_report"]
		if arg["pathologie"] is None:
			self.Pathology = ""
		else:
			self.Pathology = arg["pathologie"]
			if "-" in self.Pathology:
				self.Pathology.replace("-", " ")



class CNV:
    """Generates an object with variant attributes from Labkey CNV à vérifier"""
    def __init__(self, arg):
		self.Gene = arg["gene"]
		self.Copystatus = ""
		
		# manage copy number
#		if( arg["copy_number"] is not None:
		if arg["copy_number"] == "1":
			self.Copystatus =  "Deletion hétérozygote"
		elif arg["copy_number"] == "2":
			self.Copystatus = "Diploïdie" 
		elif arg["copy_number"] == "0":
			self.Copystatus = "Deletion homozygote"         
		elif arg["copy_number"] == "3":
			self.Copystatus =  "Duplication" 
		elif arg["copy_number"] == "4":
			self.Copystatus =  "Triplication"
		elif arg["copy_number"] == "5":
			self.Copystatus =  "Quadriplication"
		elif arg["copy_number"] is None:
			sys.stderr.write( "\tNo copy number level found in Labkey for the sample\n" )
		
		
		self.Mut = "<br/>".join( [arg["interval_target"] , self.Copystatus] )
		self.Inherit = arg["transmission"]
		self.Classification = arg["classification"]
		self.ToReport = arg["to_report"]
		if arg["pathologie"] is None:
			self.Pathology = ""
		else:
			self.Pathology = arg["pathologie"]
			if "-" in self.Pathology:
				self.Pathology.replace("-", " ")
























