#!/usr/bin/python
# -*- coding: utf-8 -*-

### GAD PIPELINE ###
## organize_data_folder.py
## Version : 2.4.0
## Description :
## Usage : organize_data_folder.py <-d input directory> [-o output file] [-p pipelinebase] [-e log path]
## Output : a bash script ready to be qsub for organizing folder, list files for family, and variant calling pattern
## Requirements : python 2.7+, labkey API installed

## Author : yannis.duffourd@u-bourgogne.fr
## Creation Date : 2015-07-15
## last revision date : 20181119
## Known bugs :
#


import os
import sys
import glob
import getopt
import logging
from LabkeyTools import GetRows
from LabkeyTools import PatientReport


inputdir = ''
logpath = ''

specimenRepo = {}
targetTable = {}
sampleList = []
allPedList = []

try:
	opts, args = getopt.getopt(sys.argv[1:], 'd:i:o:p:t:e:')
	for opt, arg in opts:
		if opt in ("-d"):
			inputdir = arg
		elif opt in ("-o"):
			sys.stdout = open(arg, 'w')
		elif opt in ("-p"):
			pipelinepath = arg
		elif opt in ("-t"):
			targetpath = arg
		elif opt in ("-e"):
			logpath = arg
except getopt.GetoptError:
	print "usage : python organize_data_folder.py <-d input directory> <-t targetpath> <-p pipelinepath> [-o output file] [-e log file]'"
	sys.exit(2)

if inputdir == "/" or inputdir == "//" or inputdir == "":
	 print "Warning : your input directory : %s seems dangerous ! Stopping execution." % inputdir
	 sys.exit(1)


if logpath == "":
	logpath = inputdir+"/organize_data_folder.log"


# logging
logging.basicConfig(filename = '%s' % (logpath), filemode = 'a', level = logging.INFO, format = '%(asctime)s %(levelname)s - %(message)s')
logging.info('start')

# does input dir exist ?
if not os.path.isdir(inputdir):
	logging.exception('The input directory does not exist : stopping execution')
	sys.exit(1)
# is the dir emtpy ?
temp_dict = os.listdir(inputdir)
if not len(temp_dict) > 0:
	logging.exception('The input directory is empty : stopping execution')
	sys.exit(1)

fastqFound = 0

# ls directory to have dijex name
for dirfile in sorted(os.listdir(inputdir)):
	if dirfile.endswith( "fastq.gz" ):
		# get the sample name
		tmp = dirfile.split( "." )
		sampleName = tmp[0]
		# add it to a list
		sampleList.append( sampleName )
		fastqFound += 1


# case where no fastq file were found, so trying directory
dirFound = 0
if fastqFound == 0:
	logging.exception('The input directory does not contain any fastq file : trying to detect sample from directories')
	for dirfile in sorted(os.listdir(inputdir)):
		if os.path.isdir( dirfile ):
			logging.info( "Found a directory : " + dirfile )
			sampleList.append( dirfile )
			dirFound += 1
			# check if log and QC subdirectories are created.
			if not os.path.isdir( dirfile + "/logs" ):
				os.mkdir( dirfile + "/logs" )
				logging.info( "Creating " + dirfile + "/logs" )
			if not os.path.isdir( dirfile + "/QC" ):
				os.mkdir( dirfile + "/QC" )
				logging.info( "Creating " + dirfile + "/QC" )



if dirFound == 0 and fastqFound == 0:
	logging.exception('The input directory does not contain any subdirectories : stopping execution : Please provide at least one sample.')
	sys.exit(1)




# sort and uniq the list
sampleList = list( set( sampleList ) )

case = "nothing"


# test wether it's miseq or wes
if sampleList[0].startswith( "dije")  or sampleList[0].startswith( "pool"):
	#it's wes
	case = "wes"
elif sampleList[0].startswith( "dijarn"):
	case = "rnaseq"
else:
	# it's miseq
	case = "miseq"

if case == "nothing":
	logging.exception('fastq file not detected in the provided input directory : stopping execution')
	sys.exit(1)


if case == "miseq":
	logging.info( "Getting information for miseq run" )

	# init tables
	familyPedList = []
	gadList = sampleList

	#  get the batchname : it's where we are : to modify
	pathTable = inputdir.split( "/" )
	if inputdir.endswith( "/" ):
		batch = pathTable[len(pathTable) - 2]
	else:
		batch = pathTable[len(pathTable) - 1]

	logging.info( "batch name : " + str( batch ) )

	allMiseqRun = GetRows( "MiSeq run" )["rows"]
	for l in allMiseqRun:
		if l["run_name"] == batch:
			for x in sampleList:
			#print "Looking for %s in sample list ( %s )" % ( l["SpecimenID"], x )
				if str(l["SpecimenID"]) == str(x):
					#print "Found !"
					# get informations
					PED = str(l["PatientID"])
					genes = str( l["Description"] )
					specimenRepo[ str(l["SpecimenID"]) ] = PED
					targetTable[ str(l["SpecimenID"]) ] = genes
					logging.info( "Info from MiSeq run dataset : PED = " + PED + " ; GAD = " + str( l["SpecimenID"] ) + " ; target = " + genes )

					j = PED.split( "." )[0]
					if j in allPedList:
						familyPedList.append( j )
					else :
						allPedList.append( j )


	familyPedList = list( set( familyPedList ) )
	for i in familyPedList:
		if os.path.isfile( str(i) + ".family.list" ):
			os.remove( str(i) + ".family.list" )
			logging.info( "Deleting PED file " + str(i) + ".family.list to recalculate family members" )
		logging.info( "Checking family : " + str(i) )
		writeStream = open( str(i) + ".family.list" , 'a' )
		for a in specimenRepo:
			b = specimenRepo[a].split( "." )
			if b[0] == i:
				logging.info( "found a specimen : " + str(a) )
				writeStream.write( str(a) + "\n" )
		writeStream.close()


PEDlist, AllPEDlist = [], []
if case == "wes" or case == "rnaseq":

	# Create family.list file
	PEDdict = {}
	dijexlist = []
	for s in sampleList:
		if s.startswith( 'dij' ):
			dijexlist.append(s)
	allWESRun = GetRows( "Suivi exomes" )["rows"]
	for l in allWESRun:
		if l["dijexID"] in sampleList:
			PED = str(l["PatientID"]).split(".")[0]
			PEDdict.setdefault(PED, []).append(l["dijexID"])
			AllPEDlist.append(PED)
		if l["dijexID"] in dijexlist:
			PEDlist.append(PED)
	PEDlist = list(set(PEDlist))
	AllPEDlist = list(set(AllPEDlist))
	for p in AllPEDlist:
		if os.path.isfile(str(p)+".family.list"):
			os.remove(str(p)+".family.list")
			logging.info( "Deleting PED file "+str(p)+".family.list to recalculate family members" )
		logging.info("Checking family : "+str(p))
		writeStream = open(str(p)+".family.list" , 'a' )
		for d in list(set(PEDdict[p])):
			logging.info( "found a specimen : "+str(d))
			writeStream.write(str(d)+"\n")
		writeStream.close()



# Now we have a dict of dijexid / gadid associated to their pedid.
# From now, we have all information needed. we will configure the analysis steps which recquire multisample config.

# Construct the analysis folder
if ( case != "nothing" and fastqFound > 0 ):
	for i in sampleList:
		logging.info( "Organizing sample " + str(i) )
		# creating dir
		if not os.path.isdir( str(inputdir) + "/" + str(i)  ):
			os.mkdir( str(inputdir) + "/" + str(i) )
			logging.info( "Creating " + str(inputdir) + "/" + str(i) )
			os.mkdir( str(inputdir) + "/" + str(i) + "/QC" )
			logging.info( "Creating " + str(inputdir) + "/" + str(i) + "/QC" )
			os.mkdir( str(inputdir) + "/" + str(i) + "/logs" )
			logging.info( "Creating " + str(inputdir) + "/" + str(i) + "/logs" )
		else:
			logging.warning( "Folder : " + str(inputdir) + "/" + str(i) + " already exists : continuing ")



		# moving R1 fastq
		if not os.path.isfile( inputdir + "/" + i + "/" + i + ".R1.fastq.gz" ):
			os.rename( str(inputdir) + "/" + str(i) + ".R1.fastq.gz" , str(inputdir) + "/" + str(i) + "/" + str(i) + ".R1.fastq.gz" )
			logging.info( "Moving " + str(inputdir) + "/" + str(i) + ".R1.fastq.gz TO " + str(inputdir) + "/" + str(i) + "/" + str(i) + ".R1.fastq.gz" )
		else:
			logging.exception( "File : " + str(inputdir) + "/" + str(i) + "/" + str(i) + ".R1.fastq.gz already exists ! : stopping execution" )
			sys.exit( 1 )

		# moving R2 fastq
		if not os.path.isfile( inputdir + "/" + i + "/" + i + ".R2.fastq.gz" ):
			os.rename( str(inputdir) + "/" + str(i) + ".R2.fastq.gz" , str(inputdir) + "/" + str(i) + "/" + str(i) + ".R2.fastq.gz" )
			logging.info( "Moving " + str(inputdir) + "/" + str(i) + ".R2.fastq.gz TO " + str(inputdir) + "/" + str(i) + "/" + str(i) + ".R2.fastq.gz" )
		else:
			logging.exception( "File : " + str(inputdir) + "/" + str(i) + "/" + str(i) + ".R2.fastq.gz already exists ! : stopping execution" )
			sys.exit( 1 )



writtenSamples = []



# getting informations for each sample
# patient related informations
allDnatheque = GetRows( "DNAThèque" )["rows"]
for i in sampleList:

	logging.info( "Getting informations for sample " + str(i) )
	if os.path.isfile( str(inputdir) + "/" + str(i) + "/" + str(i) + ".info" ):
		os.remove( str(inputdir) + "/" + str(i) + "/" + str(i) + ".info" )
		logging.info( "Deleting info file par patient " + str(i) + " to get fresh informations" )

	outfile = open( str(inputdir) + "/" + str(i) + "/" + str(i) + ".info" , "w" )
	outfile.write( "sampleId\t%s\n" % i )

	if case == "wes" or case == "rnaseq":
		for l in allWESRun:
			if str(l["dijexID"]) == i:
				gad = str(l["SpecimenID"])
				if case != "rnaseq" :
					l["capture"] = l["capture"].replace(" ", "_")
					if len(glob.glob(targetpath+"/"+l["capture"]+".50bp.*.interval_list")) == 1:
						target = glob.glob(targetpath+"/"+l["capture"]+".50bp.*.interval_list")[0]
					else:
						target = glob.glob(targetpath+"/Agilent_CRE.50bp.*.interval_list")[0]
						logging.info('Target file not found for '+l["capture"]+': use Agilent_CRE target file')
				else:
					# if rnaseq, use the refseq target.
					target = glob.glob(targetpath+"/refseq.cds.2bp.hg19.with_genes.interval_list")[0]
				PED = l["PatientID"]
				for l in allDnatheque:
					if str(l["sampleID"]) == gad:
						sex = str(l["DataSets/Patients/sexe"])
						parente = str(l["DataSets/Patients/parente"])
						statut = str(l["DataSets/Patients/statut"])
						autre = str(l["sampleID/numero_autre"])
						origine = str(l["sampleID/origine_de_l_echantillon"])

	else:
		gad = i
		logging.info("Current GAD number : " + str( gad ) + " : target : " + str(targetTable[ gad ]) )
		targetToSend = ""
		first = 0
		if targetTable[ gad ] == "Panel_Mosaique_V1":
			target = "/work/gad/shared/pipeline/hg19/targets/panel_mosaique_v1.targets.interval_list"
		else:
			tmpTargetTable = targetTable[ gad ].split( "_" )
			
			for a in tmpTargetTable:
				a = a.replace( " " , "" )
				if ( a != "vali" ) and ( len( a ) > 1 ):
					if not first == 0:
						targetToSend += ","
					targetToSend += str(a)
					first += 1
			logging.info( "Target option to send : " + str(targetToSend) )
			# define target files
			outfile_target = str(inputdir) + "/" + str(i) + "/target.interval_list"
			out_stream = open( outfile_target , "w" )
			out_stream.close()

			#get_interval_list_from_genes.py
			sys.argv = [ "get_interval_list_from_genes.py" , "-g" , targetToSend , "-i" , "/work/gad/shared/pipeline/hg19/targets/refseq.cds.2bp.hg19.with_genes.interval_list" , "-o" , outfile_target ]
			# to modify
			execfile(pipelinepath+"/common/intervals/get_interval_list_from_genes.py")
			target = outfile_target

		for l in allDnatheque:
			if str(l["sampleID"]) == gad:
				PED = str(l["PatientID"])
				sex = str(l["DataSets/Patients/sexe"])
				parente = str(l["DataSets/Patients/parente"])
				statut = str(l["DataSets/Patients/statut"])
				autre = str(l["sampleID/numero_autre"])
				origine = str(l["sampleID/origine_de_l_echantillon"])


	outfile = open( str(inputdir) + "/" + str(i) + "/" + str(i) + ".info" , "w" )
	outfile.write( "PatientId\t%s\n" % PED )
	outfile.write( "GAD\t%s\n" % gad )
	outfile.write( "Sex\t%s\n" % sex )
	outfile.write( "Parente\t%s\n" % parente )
	outfile.write( "Statut\t%s\n" % statut )
	outfile.write( "Numero autre\t%s\n" % autre )
	outfile.write( "Origine\t%s\n" % origine )
	outfile.write( "Path to target\t%s\n" % target )
	outfile.close()



# Check for missing samples in trio

missingList = {}
if case == "wes" or case == "rnaseq":
	# Check for missing samples
	for l in allWESRun:
		if l["PatientID"].split( "." )[0] in AllPEDlist and l["dijexID"] not in sampleList and l["statut"] != "Echec CQ" and l["statut"] != "Annulé" and l["statut"] != "Envoyé" and (l["dijexID"].startswith( "dijex") or l["dijexID"].startswith( "pool")):
			missingList[l["dijexID"]] = {"PED":l["PatientID"],"gad":str(l["SpecimenID"])}
			if case != "rnaseq" :
				l["capture"] = l["capture"].replace(" ", "_")
				if len(glob.glob(targetpath+"/"+l["capture"]+".50bp.*.interval_list")) == 1:
					target = glob.glob(targetpath+"/"+l["capture"]+".50bp.*.interval_list")[0]
				else:
					target = glob.glob(targetpath+"/Agilent_CRE.50bp.*.interval_list")[0]
			else:
				target = glob.glob(targetpath+"/refseq.cds.2bp.hg19.with_genes.interval_list")[0]
			missingList[l["dijexID"]]["target"] = target
	# Get informations for missing samples
	for l in allDnatheque:
		for d in missingList.keys():
			if str(l["sampleID"]) == missingList[d]["gad"]:
				missingList[d]["sex"] = str(l["DataSets/Patients/sexe"])
				missingList[d]["parente"] = str(l["DataSets/Patients/parente"])
				missingList[d]["statut"] = str(l["DataSets/Patients/statut"])
				missingList[d]["autre"] = str(l["sampleID/numero_autre"])
				missingList[d]["origine"] = str(l["sampleID/origine_de_l_echantillon"])
	# Create bash script
	if len(missingList.keys()) > 0:
		scriptfile = open( str(inputdir) + "/check_trio.sh" , "w" )
		scriptfile.write("#!/bin/bash\n")
		for d in missingList.keys():
			scriptfile.write("# Missing sample: "+d+" for: "+missingList[d]["PED"].split( "." )[0]+"\n")
			scriptfile.write("mkdir "+inputdir+"/"+d+"/\n")
			scriptfile.write("mkdir "+inputdir+"/"+d+"/logs\n")
			scriptfile.write("mkdir "+inputdir+"/"+d+"/QC\n")
			scriptfile.write("cp /archive/gad/shared/bam_new_genome_temp/"+d+".bam "+inputdir+"/"+d+"/\n")
			scriptfile.write("cp /archive/gad/shared/bam_new_genome_temp/"+d+".bai "+inputdir+"/"+d+"/\n")
			scriptfile.write("echo "+d+" >> "+inputdir+"/"+missingList[d]["PED"].split( "." )[0]+".family.list"+"\n")
			scriptfile.write("echo -e \"PatientId\\t"+missingList[d]["PED"]+"\" > "+inputdir+"/"+d+"/"+d+".info"+"\n")
			scriptfile.write("echo -e \"GAD\\t"+missingList[d]["gad"]+"\" >> "+inputdir+"/"+d+"/"+d+".info"+"\n")
			scriptfile.write("echo -e \"Sex\\t"+missingList[d]["sex"]+"\" >> "+inputdir+"/"+d+"/"+d+".info"+"\n")
			scriptfile.write("echo -e \"Parente\\t"+missingList[d]["parente"]+"\" >> "+inputdir+"/"+d+"/"+d+".info"+"\n")
			scriptfile.write("echo -e \"Statut\\t"+missingList[d]["statut"]+"\" >> "+inputdir+"/"+d+"/"+d+".info"+"\n")
			scriptfile.write("echo -e \"Numero autre\\t"+missingList[d]["autre"]+"\" >> "+inputdir+"/"+d+"/"+d+".info"+"\n")
			scriptfile.write("echo -e \"Origine\\t"+missingList[d]["origine"]+"\" >> "+inputdir+"/"+d+"/"+d+".info"+"\n")
			scriptfile.write("echo -e \"Path to target\\t"+missingList[d]["target"]+"\" >> "+inputdir+"/"+d+"/"+d+".info"+"\n")
