#!/bin/bash

### GAD PIPELINE ###
## process_RNASeq_alignement_2pass.sh
## Description : Align reads with BWA
## Usage : qsub -v INPUTFILEONE=</path/to/the/fastq/R1/file>,INPUTFILETWO=</path/to/the/fastq/R2/file>,[LOGFILE=/path/to/the/log/file],[CONFIGFILE=/path/to/the/config/file] process_RNASeq_alignement_2pass.sh
## Output : unsorted BAM file
## Requirements : STAR2

## Author : yannis.duffourd@u-bourgogne.fr
## Creation Date : 20170327
## last revision date : 20200929
## Known bugs : None

#$ -V
#$ -q batch


# logfile
if [ -z ${LOGFILE} ]
then
    LOGFILE=process_star_alignement.$(date +"%F_%H-%M-%S").log
fi

# Logging
exec 1>> $LOGFILE 2>&1
echo "$(date +"%F_%H-%M-%S"): START"

# Check if input file exist
if [ ! -f $INPUTFILEONE ]
then
    echo "Input file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch star2_alignement.failed
    exit 1
fi

# Check if input file exist
if [ ! -f $INPUTFILETWO ]
then
    echo "Input file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch star2_alignement.failed
    exit 1
fi

# Config file path option
if [ -z ${CONFIGFILE} ]
then
    CONFIGFILE=analysis_config.tsv
fi


SAMPLE=$(basename ${INPUTFILEONE%%.*})
STAR=$(grep "star2" $CONFIGFILE | awk '{print $2}')
PATHSCRIPT=$(grep "pipelinebase" $CONFIGFILE | awk '{print $2}')
REFDIR=$(grep "refdir" $CONFIGFILE | awk '{print $2}')
GTFANNOT=$(grep "hg19gtf" $CONFIGFILE | awk '{print $2}')


echo "command : $STAR/STAR \
	--runThreadN $NSLOTS \
	--genomeDir $REFDIR \
	--twopassMode Basic \
	--sjdbGTFfile $GTFANNOT \
	--readFilesIn $INPUTFILEONE $INPUTFILETWO \
	--readFilesCommand zcat \
	--outFileNamePrefix $SAMPLE/$SAMPLE. \
	--quantMode GeneCounts \
	--outSAMtype BAM SortedByCoordinate "


$STAR/STAR \
	--runThreadN $NSLOTS \
	--genomeDir $REFDIR \
	--twopassMode Basic \
	--sjdbGTFfile $GTFANNOT \
	--readFilesIn $INPUTFILEONE $INPUTFILETWO \
	--readFilesCommand zcat \
	--outFileNamePrefix $SAMPLE/$SAMPLE. \
	--quantMode GeneCounts \
	--outSAMtype BAM SortedByCoordinate

process_star_alignement_code=$?
echo "process_RNASeq_alignement.sh exit code : $process_star_alignement_code"

if [ $process_star_alignement_code != 0 ]
then
    echo "$(date +"%F_%H-%M-%S"): END"
    touch star2_alignement.failed
    exit 1
fi

echo "$(date +"%F_%H-%M-%S"): END"
