#!/bin/bash

### GAD PIPELINE ###
## wrapper_bam2junc.sh
## Description : compute bam into junction file for leafcutter splice analysis
## Usage : qsub -v INPUTFILE=</path/to/the/bam/file>,OUTPUTFILE=</path/to/the/junc/file>,[LOGFILE=/path/to/the/log/file],[CONFIGFILE=/path/to/the/config/file] wrapper_bam2junc.sh
## Output : junction files
## Requirements : LeafCutter, Perl v5

## Author : emilie.tisserant@u-bourgogne.fr, yannis.duffourd@u-bourgogne.fr
## Creation Date : 20180417
## last revision date : 20200929
## Known bugs : None

#$ -V
#$ -q batch
#$ -pe smp 1 


# logfile
if [ -z ${LOGFILE} ]
then
    LOGFILE=bam2junc.$(date +"%F_%H-%M-%S").log
fi

# Logging
exec 1>> $LOGFILE 2>&1
echo "$(date +"%F_%H-%M-%S"): START"

# Check if input file exist
if [ ! -f $INPUTFILE ]
then
    echo "Input file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch bam2junc.failed
    exit 1
fi

# Config file path option
if [ -z ${CONFIGFILE} ]
then
    CONFIGFILE=analysis_config.tsv
fi


sample=`basename ${INPUTFILE%%.*}`
SAMTOOLSDIR=$(grep "samtools" $CONFIGFILE | awk '{print $2}')
leafCutterDir=$(grep "leafcutterdir" $CONFIGFILE | awk '{print $2}')
PYTHONBIN=$(grep "pythonbin" $CONFIGFILE | awk '{print $2}')



bedfile=$INPUTFILE.bed

echo "Command : $SAMTOOLSDIR/samtools view $INPUTFILE | $PYTHONBIN $leafCutterDir/scripts/filter_cs.py | $leafCutterDir/scripts/sam2bed.pl --use-RNA-strand - $bedfile"
$SAMTOOLSDIR/samtools view $INPUTFILE | $PYTHONBIN $leafCutterDir/scripts/filter_cs.py | $leafCutterDir/scripts/sam2bed.pl --use-RNA-strand - $bedfile
bam2junc_code=$?
echo "wrapper_bam2junc.sh exit code : $bam2junc_code"
if [ $bam2junc_code != 0 ]
then
    echo "$(date +"%F_%H-%M-%S"): END"
    touch bam2junc.failed
    exit 1
fi

echo "Command:$leafCutterDir/scripts/bed2junc.pl $bedfile $OUTPUTFILE"
$leafCutterDir/scripts/bed2junc.pl $bedfile $OUTPUTFILE
bam2junc_code=$?
echo "wrapper_bam2junc.sh exit code : $bam2junc_code"
if [ $bam2junc_code != 0 ]
then
    echo "$(date +"%F_%H-%M-%S"): END"
    touch bam2junc.failed
    exit 1
fi

echo "Command: rm $bedfile"
rm $bedfile

echo "$(date +"%F_%H-%M-%S"): END"
