#!/bin/bash

### GAD PIPELINE ###
## wrapper_differential_intron_analysis_rare.sh
## Description : perform differential analysis of outlier rare splice from leafcutter analysis
## Usage : qsub -v INPUTFILE=<intron_count_file>,DESIGN=<design_file>,[OUTPUTPREFIX=<outputprefix>],[LOGFILE=/path/to/the/log/file],[CONFIGFILE=/path/to/the/config/file] wrapper_differential_intron_analysis.sh
## Output :
## Requirements : LeafCutter, R 3.x

## Author : emilie.tisserant@u-bourgogne.fr, yannis.duffourd@u-bourgogne.fr
## Creation Date : 20180420
## last revision date : 20201005
## Known bugs : None

#$ -V
#$ -q batch



# logfile
if [ -z ${LOGFILE} ]
then
    LOGFILE=differential_intron_analysis.$(date +"%F_%H-%M-%S").log
fi

# Logging
exec 1>> $LOGFILE 2>&1
echo "$(date +"%F_%H-%M-%S"): START"

# Check if input file exist
if [ ! -f $INPUTFILE ]
then
    echo "Input file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch differential_intron_analysis.failed
    exit 1
fi

# Check if input file exist
if [ ! -f $DESIGN ]
then
    echo "Design file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch differential_intron_analysis.failed
    exit 1
fi

# Check outputprefix
if [ -z ${OUTPUTPREFIX} ]
then
    OUTPUTPREFIX=""
else
    OUTPUTPREFIX="-o $OUTPUTPREFIX"
fi

# Config file path option
if [ -z ${CONFIGFILE} ]
then
    CONFIGFILE=analysis_config.tsv
fi


# load the dedicated environnement for R install
leafCutterDir=$(grep "leafcutterdir" $CONFIGFILE | awk '{print $2}')
EXONFILE=$(grep "leafcutterannot" $CONFIGFILE | awk '{print $2}')
PIPELINEBASE=$(grep "pipelinebase" $CONFIGFILE | cut -f2)
source activate leafcutter
source $PIPELINEBASE/common/splice/load_leafcutter_env.sh


echo "Command : $leafCutterDir/scripts/leafcutter_ds.R -e $EXONFILE -i 1 -g 1 -c 3 $OUTPUTPREFIX $INPUTFILE $DESIGN"
$leafCutterDir/scripts/leafcutter_ds.R -e $EXONFILE -i 1 -g 1 -c 3 $OUTPUTPREFIX $INPUTFILE $DESIGN
exit_code=$?
echo "differential_intron_analysis.sh exit code : $exit_code"
if [ $exit_code != 0 ]
then
    echo "$(date +"%F_%H-%M-%S"): END"
    touch differential_intron_analysis.failed
    exit 1
fi


echo "$(date +"%F_%H-%M-%S"): END"
