#!/bin/bash

### GAD PIPELINE ###
## wrapper_format_leafcutter.sh
## Description : annotate a report file (from wgs analysis) with rnaseq analysis results
## Usage : qsub -v INPUTFILE=</path/to/the/leafcutter/outlier_cluster_significance/file>,OUTPUTFILE=</path/to/the/leafcutter/formated/output/file>,[LOGFILE=/path/to/the/log/file],[CONFIGFILE=/path/to/the/config/file] wrapper_format_leafcutter.sh
## Output : leafcutter final result file (tsv)
## Requirements : python 2.7, leafcutter result, OMIM file

## Author : yannis.duffourd@u-bourgogne.fr
## Creation Date : 20201021
## last revision date : 20201213
## Known bugs : None

#$ -V
#$ -q batch

# logfile
if [ -z ${LOGFILE} ]
then
    LOGFILE=format_leafcutter.$(date +"%F_%H-%M-%S").log
fi

# Logging
exec 1>> $LOGFILE 2>&1
echo "$(date +"%F_%H-%M-%S"): START"

# Check if input file exist
if [ ! -f $INPUTFILE ]
then
    echo "Input file does not exist"
    echo "$(date +"%F_%H-%M-%S"): END"
    touch format_leafcutter.failed
    exit 1
fi

# Config file path option
if [ -z ${CONFIGFILE} ]
then
   CONFIGFILE=analysis_config.tsv
fi

PYTHONBIN=$(grep "pythonbin" $CONFIGFILE | awk '{print $2}')
PIPELINEBASE=$(grep "pipelinebase" $CONFIGFILE | cut -f2)
OMIM=$(grep "omim_list" $CONFIGFILE | cut -f2)
samplebase=${INPUTFILE%%.*}

# adding coordinates to leafcutter results
TMPOUTFILE=$samplebase.leafcutter.format.tsv
echo "Command : $PYTHONBIN $PIPELINEBASE/common/splice/format_leafcutter.py -i $INPUTFILE -e $samplebase.leafcutter.outlier_effect_sizes.txt -o $TMPOUTFILE "
$PYTHONBIN $PIPELINEBASE/common/splice/format_leafcutter.py -i $INPUTFILE -e $samplebase.leafcutter.outlier_effect_sizes.txt -o $TMPOUTFILE
exit_code=$?
echo "wrapper_format_leafcutter.sh exit code : $exit_code"

if [ $exit_code != 0 ]
then
    echo "$(date +"%F_%H-%M-%S"): END"
    touch format_leafcutter.failed
    exit 1
fi

# omim annotation
TMPINFILE=$TMPOUTFILE
TMPOUTFILE=$samplebase.leafcutter.format.omim.tsv
echo "Command : $PYTHONBIN $PIPELINEBASE/common/splice/annotate_leafcutter_events.py -i $TMPINFILE -d $OMIM -o $TMPOUTFILE"
$PYTHONBIN $PIPELINEBASE/common/splice/annotate_leafcutter_events.py -i $TMPINFILE -d $OMIM -o $TMPOUTFILE
exit_code=$?
echo "wrapper_format_leafcutter.sh exit code : $exit_code"

if [ $exit_code != 0 ]
then
    echo "$(date +"%F_%H-%M-%S"): END"
    touch format_leafcutter.failed
    exit 1
fi

# filter and produce the final report
echo "Command : grep '^cluster' $TMPOUTFILE > $OUTPUTFILE && awk -F '\t' '($6<0.05){print $0}' $TMPOUTFILE >> $OUTPUTFILE"
grep "^cluster" $TMPOUTFILE > $OUTPUTFILE
awk -F "\t" '($6<0.05){print $0}' $TMPOUTFILE >> $OUTPUTFILE
exit_code=$?
echo "wrapper_format_leafcutter.sh exit code : $exit_code"

if [ $exit_code != 0 ]
then
    echo "$(date +"%F_%H-%M-%S"): END"
    touch format_leafcutter.failed
    exit 1
fi

echo "$(date +"%F_%H-%M-%S"): END"
